#include <QDir>
#include <QString>
#include <QStringList>
#include <QDate>
#include <QDialog>
#include <QIcon>
#include <QLocale>
#include <QTranslator>
#include <QAction>

#include <QDebug>
#include <QCoreApplication>

#include "ConnectionLogModule.h"

static QTranslator *translator = 0;

extern "C" void module_init(Applet *applet)
{
    qDebug( "ConnectionLogModule: module_init." );

    QString translatorFileName = QLatin1String( "LogViewer_" );
    translatorFileName += QLocale::system().name();

    translator = new QTranslator();

    if ( !translator->load( translatorFileName, QString( QString( PKG_PREFIX ).remove( "\"" ) + "/share/ubudsl/translations/" ) ) )
    {
        qDebug() << "Failed to load the" << QString( QString( PKG_PREFIX ).remove( "\"" ) + "/share/ubudsl/translations/" + translatorFileName + "!" );
    }

    QCoreApplication::instance()->installTranslator( translator );

    connectionLogModule = new ConnectionLogModule(applet);
}

extern "C" void module_exit()
{
    qDebug( "ConnectionLogModule: module_exit." );

    delete connectionLogModule;
    connectionLogModule = 0;

    if ( translator != NULL )
    {
        QCoreApplication::instance()->removeTranslator( translator );
        delete translator;
        translator = 0;
    }
}

typedef QList<ConnectionLogModule::LogEvent> LogEventList; 
typedef ConnectionLogModule::ConnectionData  ConnectionData; 

const QDir ConnectionLogModule::logDir( "/var/log/ubudsl" );

QRegExp ConnectionLogModule::nameRegExp( "^([0-9]{2})-([0-9]{2})-([0-9]{4})\\.log$" );
QRegExp ConnectionLogModule::ipRegExp( "^Registered IP:\\s+([0-9]{1,4}.[0-9]{1,4}.[0-9]{1,4}.[0-9]{1,4}.)$" );

ConnectionData &ConnectionData::operator+=( const ConnectionData &other )
{
    this->rx += other.rx;
    this->tx += other.tx;

    return *this;
}

ConnectionLogModule::ConnectionLogModule( Applet *parent )
:applet( parent ),
 m_menuAction( new QAction( QIcon( ":icons/log_viewer.png" ), tr( "Connection Log" ), this ) ),
 m_fsWatcher( new QFileSystemWatcher( this ) )
{
    foreach ( QString file, logDir.entryList( QStringList( "*.log" ) ) )
        m_fsWatcher->addPath( logDir.absoluteFilePath( file ) );

    refreshLogFiles();

    ui.setupUi( &m_window );

    connect( m_menuAction, SIGNAL( triggered() ), &m_window, SLOT( show() ) );
    connect( m_fsWatcher, SIGNAL( directoryChanged( const QString & ) ), this, SLOT( logDirChanged() ) );
    connect( m_fsWatcher, SIGNAL( fileChanged( const QString & ) ), this, SLOT( forceUpdate() ) );
    connect( ui.calendar, SIGNAL( currentPageChanged( int, int ) ), this, SLOT( calendarPageChanged( int, int ) ) );
    connect( ui.calendar, SIGNAL( selectionChanged() ), this, SLOT( dateChanged() ) );

    dateChanged();

    m_window.setAttribute( Qt::WA_QuitOnClose, false );
    m_window.setWindowIcon( QIcon( QString( PKG_PREFIX ).remove( "\"" ) + "/share/pixmaps/ubudsl.png" ) );
    m_window.setWindowTitle( "UbuDSL" );

    applet->m_pluginsMenu->addAction( m_menuAction );
}

ConnectionLogModule::~ConnectionLogModule()
{
    applet->m_pluginsMenu->removeAction( m_menuAction );
    delete m_menuAction;
    delete m_fsWatcher;
}

void ConnectionLogModule::refreshLogFiles()
{
    logFiles.clear();

    foreach ( QString fileName, logDir.entryList() )
    {
        if ( nameRegExp.exactMatch( fileName ) )
        {
            QDate date( nameRegExp.cap( 3 ).toInt(), nameRegExp.cap( 2 ).toInt(), nameRegExp.cap( 1 ).toInt() );
            logFiles.insert( logDir.absoluteFilePath( fileName ), date );
        }
    }
}

LogEventList ConnectionLogModule::getEventsForDate( const QDate &date )
{
    QRegExp logLineRegExp( "^[0-9]{2}\\.[0-9]{2}\\.[0-9]{4}\\s+([0-9]{2}:[0-9]{2}:[0-9]{2}):\\s+(.+)\\s+$" );
    QFile log( logFiles.key( date ) );
    LogEventList eventList;

    if ( !log.exists() || !log.open( QFile::ReadOnly ) )
        return eventList;

    while ( !log.atEnd() )
    {
        if ( logLineRegExp.exactMatch( log.readLine() ) )
        {
            LogEvent e;
            e.time = logLineRegExp.cap( 1 );
            e.event = logLineRegExp.cap( 2 );

            eventList << e;
        }
    }

    return eventList;
}

ConnectionData ConnectionLogModule::getDataForDate( const QDate &date )
{
    ConnectionData data;

    data.rx = 0;
    data.tx = 0;

    QRegExp dataLineRegExp( "^[0-9]{2}\\.[0-9]{2}\\.[0-9]{4}\\s+[0-9]{2}:[0-9]{2}:[0-9]{2}:\\s+RX:(.+)\\sTX:(.+)\\s+$" );
    QFile log( logFiles.key( date ) );

    if ( !log.exists() || !log.open( QFile::ReadOnly ) )
        return data;

    while ( !log.atEnd() )
    {
        if ( dataLineRegExp.exactMatch( log.readLine() ) )
        {
            data.rx += dataLineRegExp.cap( 1 ).toDouble();
            data.tx += dataLineRegExp.cap( 2 ).toDouble();
        }
    }

    return data;
}

ConnectionData ConnectionLogModule::getDataForMonth( const QDate &date )
{
    ConnectionData data;

    data.rx = 0;
    data.tx = 0;

    QHashIterator<QString, QDate> i( logFiles );
    while ( i.hasNext() )
    {
        i.next();

        if ( i.value().month() == date.month() && i.value().year() == date.year() )
            data += getDataForDate( i.value() );
    }

    return data;
}

void ConnectionLogModule::calendarPageChanged( int year, int month )
{
    ui.calendar->setSelectedDate( QDate( year, month, ui.calendar->selectedDate().day() ) );
}

void ConnectionLogModule::dateChanged( bool forceSameDate )
{
    static QDate previous;
    QDate date = ui.calendar->selectedDate();

    if ( date == previous && !forceSameDate )
        return;

    ConnectionData data = getDataForDate( date );

    ui.dayDownloaded->setText( tr( "%1 MB" ).arg( data.rx/1048576, 0, 'f', 2 ) );
    ui.dayUploaded->setText( tr( "%1 MB" ).arg( data.tx/1048576, 0, 'f', 2 ) );

    if ( date.month() != previous.month() || date.year() != previous.year() || forceSameDate )
    {
        data = getDataForMonth( date );

        ui.monthDownloaded->setText( tr( "%1 MB" ).arg( data.rx/1048576, 0, 'f', 2 ) );
        ui.monthUploaded->setText( tr( "%1 MB" ).arg( data.tx/1048576, 0, 'f', 2 ) );
    }


    ui.eventsBrowser->clear();

    foreach ( LogEvent event, getEventsForDate( date ) )
    {
        if ( ipRegExp.exactMatch( event.event ) )
            ui.eventsBrowser->append( "<b>" + event.time + ":</b> " + tr( "Registered IP Address:" ) + " " + ipRegExp.cap( 1 ) + ".\n" );
        else if ( event.event == "Connection terminated" )
            ui.eventsBrowser->append( "<b>" + event.time + ":</b> " + tr( "Connection terminated." ) + "\n" );
        else if ( event.event == "Restarting modem" )
            ui.eventsBrowser->append( "<b>" + event.time + ":</b> " + tr( "Restarting modem." ) + "\n" );
        else if ( event.event == "Modem restarted" )
            ui.eventsBrowser->append( "<b>" + event.time + ":</b> " + tr( "Modem restarted." ) + "\n" );
        else if ( event.event == "Modem unplugged" )
            ui.eventsBrowser->append( "<b>" + event.time + ":</b> " + tr( "Modem unplugged." ) + "\n" );
    }

    previous = date;
}

void ConnectionLogModule::forceUpdate()
{
    qDebug("forceUpdate");
    refreshLogFiles();
    dateChanged( true );
}

void ConnectionLogModule::logDirChanged()
{
    foreach ( QString file, logDir.entryList( QStringList( "*.log" ) ) )
        m_fsWatcher->addPath( logDir.absoluteFilePath( file ) );

    forceUpdate();
}

int main( int argc, char *argv[] )
{
    QApplication app( argc, argv );
    ConnectionLogModule m(0);

    return app.exec();
}

ConnectionLogModule *connectionLogModule;

