/*
 * File: Apt.h
 * Author: Adrian Łubik (2008)
 * License: GNU LGPL v3
 */

#ifndef _APT_H_
#define _APT_H_

#include <QString>
#include <QRegExp>
#include <QStringList>
#include <QDebug>
#include <QObject>
#include <QFile>
#include <QProcess>
#include <QList>
#include <QByteArray>
#include <QListIterator>
#include <QThread>

#include "PackageManager.h"
#include "MyProcess.h"

/**
  * A class inherited from the abstarct PackageManager class.
  * Implements apt-get process handling.
  */

class Apt: public PackageManager
{
    Q_OBJECT
public:
    /**
      * Class constructor.
      * @param parent Parent's thread.
      */
    Apt( QThread *parent = 0 );
    /**
      * Class destructor.
      */
    virtual ~Apt();

    /**
      * Installs packages using the apt-get process.
      * @param pkgs List of packages to install.
      * @return Exit code of the apt-get process.
      */
    virtual int installPackages( const QStringList &pkgs );

    /**
      * Adds a CD with packages to the sources.list.
      * Uses apt-cdrom to add a CD.
      * @param mntPoint The mounting point of the CD.
      * @return Exit code of the apt-cdrom process.
      */ 
    virtual int addCD( const QString &mntPoint );

    /**
      * Checks whether the given packages are installed.
      * @param pkgs Packages to check.
      * @return List of these packages that are not installed.
      */
    virtual QStringList getNotInstalled( QStringList pkgs );

    /**
      * Checks whether the given packages are available to install.
      * Uses the apt-cache process to get informations about packages.
      * @param pkgs List of packages to check.
      * @return True if all of the given packages are available.
      */
    virtual bool getArePackagesAvailable( const QStringList &pkgs );

    /**
      * Checks whether a package is installed.
      * Uses the dpkg process to check state of the package.
      * @param pkg Package to check.
      * @return True if the package is installed.
      */
    virtual bool getIsPackageInstalled( const QString &pkg );

private slots:
    /**
      * A slot for controlling the processes' output.
      * @param ba Byte array of the process output.
      */
    void debugProcess( QByteArray ba );

private:
    MyProcess *m_cdProcess;
    MyProcess *m_installProcess;

    QProcess  *m_readListProcess;
    QProcess  *m_aptCacheProcess;

    QRegExp    m_unavailableRegExp;  //Regular expression used while debugging
                                     //to find names of not installed packages.
};

#endif
