/*
 * File: Zypper.cpp
 * Author: Adrian Łubik (2008)
 * License: GNU LGPL v3
 */

#include "Zypper.h"

Zypper::Zypper( QThread *parent )
        :Rpm( parent ),
        m_installProcess( new MyProcess( 0 ) ),
        m_zypperProcess( new MyProcess( 0 ) ),
        m_addRepoProcess( new QProcess( 0 ) ),
        m_modifyRepoProcess( new QProcess( 0 ) ),
        m_getAvailableProcess( new QProcess( 0 ) )
{
    dbg = QByteArray();

    if ( parent != NULL )
    {
        m_installProcess->moveToThread( parent );
        m_zypperProcess->moveToThread( parent );
        m_addRepoProcess->moveToThread( parent );
        m_modifyRepoProcess->moveToThread( parent );
        m_getAvailableProcess->moveToThread( parent );
    }

    QStringList env = QProcess::systemEnvironment();
    env.append( "LC_ALL=C" );

    m_installProcess->setEnvironment( env );
    m_zypperProcess->setEnvironment( env );
    m_addRepoProcess->setEnvironment( env );
    m_modifyRepoProcess->setEnvironment( env );
    m_getAvailableProcess->setEnvironment( env );

    repoRegExp.setPattern( "(\\b[0-9]{1,2}\\b)\\s*\\|\\s*(\\bYes|No\\b)\\s*\\|\\s*(\\bYes|No\\b)\\s*\\|\\s*(\\b.{1,10}\\b)\\s*\\|\\s*(\\b.{1,40}\\b)\\s*\\|\\s*(\\b.{1,40}\\b)" );

    QString readingCache( "(^. Reading repository .{1,100} cache)" );
    QString readingInstalled( "(^. Reading installed packages)" );
    QString installingPkg( "(^\\* Installing: )" );

    processing.setPattern( readingCache + "|" + readingInstalled + "|" + installingPkg );

    connect( m_installProcess, SIGNAL( lineAvailable( QByteArray ) ), this, SLOT( debugProcess( QByteArray ) ) );
}


Zypper::~Zypper()
{
    delete m_installProcess;
    delete m_zypperProcess;
    delete m_addRepoProcess;
    delete m_modifyRepoProcess;
    delete m_getAvailableProcess;
}


void Zypper::debugProcess( QByteArray ba )
{
    if ( QString( ba ).contains( processing ) )
        return;

    qDebug() << "Zypper::debugProcess():" << ba;
    dbg += ba;
}


void Zypper::debugZyppRepos( QByteArray ba )
{
    QString line = ba;

    if ( line.contains( repoRegExp ) )
    {
        Repository repo;
        repo.number  = repoRegExp.cap( 1 ).toInt();
        repo.enabled = ( repoRegExp.cap( 2 ) == "Yes" ? true : false );
        repo.refresh = ( repoRegExp.cap( 3 ) == "Yes" ? true : false );
        repo.type    = repoRegExp.cap( 4 );
        repo.alias   = repoRegExp.cap( 5 );
        repo.name    = repoRegExp.cap( 6 );

        repos.append( repo );
    }
}


int Zypper::installPackages( const QStringList &pkgs )
{
    if ( pkgs.isEmpty() )
        return 0;

    m_installProcess->clearArguments();

    m_installProcess->addArgument( "zypper" );
    m_installProcess->addArgument( "--non-interactive" );
    m_installProcess->addArgument( "--no-gpg-checks" );
// m_installProcess->addArgument( "--auto-agree-with-licenses" );
    m_installProcess->addArgument( "install" );

    foreach( QString pkg, pkgs )
    {
        m_installProcess->addArgument( pkg );
    }

    dbg += "Zypper::install(): begin.";
    m_installProcess->start();
    m_installProcess->waitForFinished( 5 * 60 * 1000 );
    dbg += "Zypper::install(): end.";

    int exitCode = m_installProcess->exitCode();
    qDebug() << "Zypper::install(): m_installProcess exited with code:" << exitCode;

    return exitCode;
}

int Zypper::addRepo( const QString &path, const QString &alias )
{
    m_addRepoProcess->start( "zypper", QStringList() << "ar" << path << alias );
    m_addRepoProcess->waitForFinished();

    return m_addRepoProcess->exitCode();
}

bool Zypper::enableRepo( QString alias, bool enable )
{
    QString success( "Repository " + alias + " has been sucessfully modified." );

    m_modifyRepoProcess->start( "zypper", QStringList() << "mr" << ( enable ? "-e" : "-d" ) << alias );
    m_modifyRepoProcess->waitForFinished();

    if ( m_modifyRepoProcess->readAll().contains( qPrintable( success ) ) )
        return true;
    else
        return false;
}

bool Zypper::getArePackagesAvailable( const QStringList &pkgs )
{
    m_getAvailableProcess->start( "zypper", QStringList() << "se" << pkgs );
    m_getAvailableProcess->waitForFinished();

    QRegExp available( "^\\s*(\\S\\s+)*\\|\\s+(.{1,100})\\s+\\|.{1,200}\\s+\\|\\s+package\\s*$" );
    QStringList tmp = pkgs;

    while ( !m_getAvailableProcess->atEnd() )
    {
        QString line = m_getAvailableProcess->readLine( 300 );
        if ( available.exactMatch( line ) && tmp.contains( available.cap( 2 ) ) )
            tmp.removeAll( available.cap( 2 ) );
    }

    return tmp.isEmpty();
}

Repositories Zypper::getRepos()
{
    m_zypperProcess->addArgument( "zypper" );
    m_zypperProcess->addArgument( "repos" );

    connect( m_zypperProcess, SIGNAL( lineAvailable( QByteArray ) ), this, SLOT( debugZyppRepos( QByteArray ) ) );

    m_zypperProcess->start();
    m_zypperProcess->waitForFinished();

    disconnect( m_zypperProcess, SIGNAL( lineAvailable( QByteArray ) ), this, SLOT( debugZyppRepos( QByteArray ) ) );

    return repos;
}

int Zypper::addCD( const QString &mntPoint )
{
    QString productFilePath;

    FileManager fm;
    fm.findFile( mntPoint, "products", productFilePath );

    if ( productFilePath.isEmpty() )
    {
        qDebug() << "product file not found, aborting";
        return -1;
    }

    QFile product( productFilePath );
    if ( product.open( QFile::ReadOnly ) )
    {
        qDebug() << "product file is open";
        QRegExp productRegExp( "^/\\s+(.{1,100})\\s*$" );
        QString line = product.readLine( 100 );

        if ( productRegExp.exactMatch( line ) )
        {
            qDebug() << "exact match:" << productRegExp.cap( 1 ).trimmed();
            Repositories repos = getRepos();
            RepositoriesIterator it( repos );

            while ( it.hasNext() )
            {
                Repository repo = it.next();
                qDebug() << "iterating, current repo:" << repo.name;
                if ( repo.name == productRegExp.cap( 1 ).trimmed() )
                {
                    if ( repo.enabled )
                    {
                        qDebug() << "repo is already enabled";
                        return 0;
                    }
                    else
                    {
                        qDebug() << "enabling repo:" << repo.alias;
                        return enableRepo( repo.alias );
                    }
                }
            }

            return addRepo( mntPoint, productRegExp.cap( 1 ).trimmed() );
        }
    }
    else
    {
        qDebug() << "product file not open, adding \"UbuDSL added CD/DVD\"";
        return addRepo( mntPoint, "UbuDSL added CD/DVD" );
    }

    return -1;
}

