/*
 * File: main.cpp
 * Author: Adrian Łubik (2008)
 * License: GNU LGPL v3
 */

#include "libubudsl/FileLocker.h"
#include "libubudsl/SystemInfo.h"
#include "libubudsl/Globals.h"

#include <unistd.h>

#include <QCoreApplication>
#include <QTranslator>
#include <QObject>
#include <QMutex>
#include <QFile>
#include <QDir>
#include <QString>
#include <QPair>
#include <QTextCodec>

#ifdef UBUDSL_GUI
# include <QApplication>
# include "UbuDSL_GUI.h"
#else
# include "UbuDSL_Cmd.h"
#endif

extern bool  debug;
extern bool  debugToFile;
extern FILE *pFile;

static QMutex mutex;
static QFile  logFile( UBUDSL_PREFIX + "/share/ubudsl/logs/UbuDSL_Configuration.log" );

void debugHandler( QtMsgType type, const char *msg );
void loadTranslation();
void showHelp();
bool checkCompatibility( QString &sys, QString &ver );
bool getIsRoot();


int main( int argc, char *argv[] )
{
    qInstallMsgHandler( debugHandler );

    QFile::remove( UBUDSL_PREFIX + "/share/ubudsl/logs/UbuDSL_Configuration.log" );
    setDebug( true, UBUDSL_PREFIX + "/share/ubudsl/logs/UbuDSL_Configuration.log" );

    QPair<int, int> virtualDeviceID = QPair<int, int>( 0, 0 );
    QRegExp virtualDeviceRegExp( "^--virtual-device(-id)*=(.{4,6}):(.{4,6})$" );

#ifndef UBUDSL_GUI
    bool openDNS = false;
#endif
    int  appExec = 0;

    for ( int i = 1; i < argc; i++ )
    {
        QString arg( argv[i] );

        if ( virtualDeviceRegExp.exactMatch( arg ) )
        {
            int vid = 0;
            int pid = 0;

            if ( virtualDeviceRegExp.cap( 2 ).startsWith( "0x" ) )
                vid = virtualDeviceRegExp.cap( 2 ).toInt( 0, 16 );
            else
                vid = QString( "0x" + virtualDeviceRegExp.cap( 2 ) ).toInt( 0, 16 );

            if ( virtualDeviceRegExp.cap( 3 ).startsWith( "0x" ) )
                pid = virtualDeviceRegExp.cap( 3 ).toInt( 0, 16 );
            else
                pid = QString( "0x" + virtualDeviceRegExp.cap( 3 ) ).toInt( 0, 16 );

            virtualDeviceID = QPair<int, int>( vid, pid );
        }

#ifndef UBUDSL_GUI
        if ( arg == "--set-opendns" )
        {
            setDebug( false );
            openDNS = true;
        }
        else
#endif
        if ( arg == "-h" || arg == "--help" )
        {
            QCoreApplication app( argc, argv );
            QTextCodec::setCodecForCStrings( QTextCodec::codecForName( "UTF-8" ) );

            loadTranslation();
            showHelp();

            return 0;
        }
    }

    //Needed for check whether dpkg file is locked
    int FD = 0;
    if ( QFile::exists( "/var/lib/dpkg/lock" ) )
        FD = lockFile( "/var/lib/dpkg/lock" );

    if ( !QFile::exists( QDir::tempPath() + "/ubudsl.lock" ) )
    {
        QFile f( QDir::tempPath() + "/ubudsl.lock" );
        f.open( QFile::WriteOnly );
        f.close();
    }
    const int UbuDSL_FD = lockFile( QDir::tempPath() + "/ubudsl.lock" );


//    QTranslator appTranslator;

#ifdef UBUDSL_GUI
    qDebug() << "UbuDSL Configurator run:" << QDateTime::currentDateTime().toString( "dd-MM-yyyy hh:mm:ss." );


    QApplication app( argc, argv );
    QTextCodec::setCodecForCStrings( QTextCodec::codecForName( "UTF-8" ) );

    app.setLayoutDirection(Qt::RightToLeft);
    QSettings palette( ":palette.ini", QSettings::IniFormat );
    app.setPalette( palette.value( "palette" ).value<QPalette>() );

    if ( !QDir::setCurrent( app.applicationDirPath() ) )
        qDebug() << "Could not set current directory as:" << app.applicationDirPath() + "/../";

    loadTranslation();

    QString sys, ver;

    if ( !checkCompatibility( sys, ver ) )
    {
        QString title = QObject::tr( "Unsupported system" );
        QString text  = QObject::tr( "Your system (%1 %2) is currently unsupported.\n\n"
                                     "Application will now quit." );
        QMessageBox::critical( 0, title, text.arg( sys, ver ) );

        qDebug() << "Unsupported system:" << sys + " " + ver;
        exit( 1 );
    }

    QString title = QObject::tr( "Critical error" );

    if ( !getIsRoot() )
    {
        QString text = QObject::tr( "UbuDSL has not been run with the administrator's "
                                    "privileges. Application will now quit." );
        QMessageBox::critical( 0, title, text );

        return -1;
    }

    if ( UbuDSL_FD < 0 )
    {
        QString text = QObject::tr( "It seems that another instance of UbuDSL is already running..." );
        QMessageBox::critical( 0, title, text );

        return -2;
    }

    if ( FD < 0 )
    {
        QString text = QObject::tr( "UbuDSL has detected a running package manager "
                                    "like apt-get or synaptic. Application can not "
                                    "work while it is running up.\n\n"
                                    "Please close all the package managers and then "
                                    "run UbuDSL again." );
        QMessageBox::critical( 0, title, text );

        return -2;
    }


    UbuDSL_GUI *gui = new UbuDSL_GUI( FD, virtualDeviceID );

    app.connect( gui, SIGNAL( quit() ), &app, SLOT( quit() ) );

    gui->move(( app.desktop()->width() - gui->width() ) / 2, ( app.desktop()->height() - gui->height() ) / 2 );
    gui->show();

    appExec = app.exec();
    delete gui;
#else
    QCoreApplication app( argc, argv );
    QTextCodec::setCodecForCStrings( QTextCodec::codecForName( "UTF-8" ) );

    if ( !QDir::setCurrent( app.applicationDirPath() ) )
        qDebug() << "Could not set current directory as:" << app.applicationDirPath() + "/../";

    loadTranslation();

    if ( openDNS )
    {
        setDebug( false );

        if ( !getIsRoot() )
            return -1;

        QStringList servers;
        servers << "208.67.222.222" << "208.67.220.220";

        DNS_Manager dns;

        if ( dns.setServers( servers ) )
        {
            QString msg = QObject::tr( "The following DNS servers have ben successfully set:\n%1" );
            printf( "%s\n", msg.arg( servers.join( "\n" ) ).toUtf8().constData() );
            return 0;
        }
        else
        {
            QString msg = QObject::tr( "An error occured while setting the OpenDNS servers!" );
            printf( "%s\n", msg.toUtf8().constData() );
            return -2;
        }
    }

    QString sys, ver;

    if ( !checkCompatibility( sys, ver ) )
    {
        qDebug() << "Unsupported system:" << sys + " " + ver;
        exit( 1 );
    }

    if ( !getIsRoot() )
        return -1;

    if ( UbuDSL_FD < 0 )
    {
        QString msg = QObject::tr( "It seems that another instance of UbuDSL is already running..." );
        fprintf( stderr, "%s\n", msg.toUtf8().constData() );
        return -2;
    }

    if ( FD < 0 )
    {
        QString msg = QObject::tr( "A working package manager was detected (apt-get or synaptic, or ...).\n"
                                   "UbuDSL can not work while package manager is running up. Please close "
                                   "it\n and then run UbuDSL again." );
        fprintf( stderr, "%s\n", msg.toUtf8().constData() );
        return -2;
    }

    UbuDSL_Cmd *cmd = new UbuDSL_Cmd( FD, virtualDeviceID );
    QObject::connect( cmd, SIGNAL( finished() ), &app, SLOT( quit() ) );

    appExec = app.exec();
    delete cmd;
#endif

    return appExec;
}

bool checkCompatibility( QString &sys, QString &ver )
{
//    QString cfgFile = qApp->applicationDirPath();
    QString cfgFile = UBUDSL_PREFIX + "/share/ubudsl/configs/system.ini";

    if ( !QFile::exists( cfgFile ) )
        cfgFile  = ":configs/system.ini";

    QSettings s( cfgFile, QSettings::IniFormat );
    QStringList groups = s.childGroups();

    sys = UbuDSL::SystemInfo::getDistributionName();
    ver = UbuDSL::SystemInfo::getDistributionRelease();

    return groups.contains( sys + "_" + ver );
}

void debugHandler( QtMsgType type, const char *msg )
{
    if ( type == QtFatalMsg )
        printf( "Fatal: " );
#ifdef UBUDSL_GUI
    else
        printf( "Debug: " );

    printf( "%s\n", msg );
#endif

    mutex.lock();

    if ( logFile.open( QIODevice::WriteOnly|QIODevice::Append ) )
    {
        QByteArray dateArray;
        dateArray.append( QDateTime::currentDateTime().toString( "dd-MM-yyyy hh:mm:ss " ) );

        logFile.write( dateArray );

        if ( type == QtFatalMsg )
            logFile.write( "UbuDSL_Configuration::debugHandler (fatal): " );
        else
            logFile.write( "UbuDSL_Configuration::debugHandler: " );

        logFile.write( msg );
        logFile.write( "\n" );
        logFile.close();
    }

    mutex.unlock();

    if ( type == QtFatalMsg )
        abort();
}

void loadTranslation()
{
    QString translatorFileName = QLatin1String( "ubudsl_" );
    translatorFileName += QLocale::system().name();

    QTranslator *translator = new QTranslator( qApp );

    if ( !translator->load( translatorFileName, QString( UBUDSL_PREFIX + "/share/ubudsl/translations/" ) ) )
    {
        qDebug() << "Failed to load the" << QString( UBUDSL_PREFIX + "/share/ubudsl/translations/" + translatorFileName + "!" );
        return;
    }

    qApp->installTranslator( translator );
}

void showHelp()
{
    QString help = QObject::tr( "UbuDSL - an ADSL connection configuration tool.\n"
                                "You can run UbuDSL with the following arguments:\n"
                                "    -h or --help:           Show this help message.\n\n"
                                "Command line version (ubudsl_cmd.bin) options:\n"
                                "          --set-opendns:    Set OpenDNS servers.\n\n"
                                "This is an Open Source software licensed under the LGPL licence.\n"
                                "(c) %1.");

    printf( "%s\n", help.arg( "Adrian Łubik 2008" ).toUtf8().constData() );
}

bool getIsRoot()
{
    int UID = getuid();

    if ( UID != 0 )
    {
        QString msg = QObject::tr( "You are not the administrator!\n"
                                   "UbuDSL must be run with the administrator's privileges!" );
        fprintf( stderr, "%s\n", msg.toUtf8().constData() );
        return false;
    }
    else
        return true;
}

