#include "ConnectThread.h"
#include "UbuDSL_Daemon.h"

//extern bool  debug;
//extern FILE *pFile;


ConnectThread::ConnectThread( QSettings *settings, int ( *fGetModemState )() )
        : getModemState( fGetModemState ),
        pppd( new MyProcess( 0 ) ),
        m_env( QProcess::systemEnvironment() ),
        m_settings( settings ),
        nas_iface( "nas0" ),
        zx_iface( "zx0" )
{
    writeLog( "ConnectThread::ConnectThread(): called." );
    qRegisterMetaType<QProcess::ProcessError>("QProcess::ProcessError");

    bool ldLibMet = false;

    for ( int i = 0; i < m_env.size(); i++ )
    {
        if ( m_env[i].startsWith( "LD_LIBRARY_PATH=" ) )
        {
            ldLibMet = true;
            m_env[i] = m_env[i].append( ":" + UBUDSL_PREFIX + "/share/ubudsl/lib" );
            break;
        }
    }

    if ( !ldLibMet )
        m_env.append( "LD_LIBRARY_PATH=" + UBUDSL_PREFIX + "/share/ubudsl/lib" );

    pppd->setEnvironment( m_env );
    pppd->moveToThread( this );
    connect( pppd, SIGNAL( lineAvailable( QByteArray ) ), this, SIGNAL( debugMsg( QByteArray ) ) );
    connect( pppd, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( pppdProcessError( QProcess::ProcessError ) ) );

    writeLog( "ConnectThread::ConnectThread(): finished." );
}

ConnectThread::~ConnectThread()
{
    writeLog( "ConnectThread::~ConnectThread(): called and finished." );
}

void ConnectThread::getPPPDArguments( const QString &fileName, QStringList &existingArgs )
{
    if ( fileName.isEmpty() || !QFile::exists(fileName) )
        return;

    QFile file(fileName);

    if ( !file.open(QFile::ReadOnly) )
        return;

    while ( !file.atEnd() )
    {
        QString line = file.readLine().trimmed();

        if ( line.startsWith("#") )
            continue;

        existingArgs << line;
    }

    file.close();
}

void ConnectThread::run()
{
    writeLog( "ConnectThread::run(): called." );
    m_returnState = ReturnState::Unknown;

    if ( !waitForSynchronisation() )
    {
        writeLog( "ConnectThread::run(): modem synchronisation failed, returning!" );
        m_returnState = ReturnState::SynchronisationFailed;
        return;
    }

    m_settings->sync();

    m_arguments.clear();
    m_arguments << "pppd" << "debug" << "linkname" << "ubudsl";

    if ( m_settings->value( "vendor_id", 0 ).toInt() == 0x0483 )
    {
        prepare_zx_iface();
        getPPPDArguments("/etc/ppp/peers/ubudsl_pppoe", m_arguments);
#ifdef PCLINUXOS
        m_arguments << "pty \"pppoe -m 1412 -I " + zx_iface + "\"";
#else
        m_arguments << "plugin rp-pppoe.so nic-" + zx_iface;
#endif
    }
    else if ( m_settings->value( "protocol", "PPPoA" ).toString() == "PPPoE" )
    {
        prepareBridging();
        prepare_nas_iface();
        getPPPDArguments("/etc/ppp/peers/ubudsl_pppoe", m_arguments);

#ifdef PCLINUXOS
        m_arguments << "pty \"pppoe -m 1412 -I " + nas_iface + "\"";
#else
        m_arguments << "plugin rp-pppoe.so nic-" + nas_iface;
#endif
    }
    else
    {
        getPPPDArguments("/etc/ppp/peers/ubudsl_pppoa", m_arguments);
    }

    if ( !m_settings->value( "pppd_arguments", QStringList() ).toStringList().isEmpty() )
        m_arguments += m_settings->value( "pppd_arguments" ).toStringList();


    qDebug() << "cmd:" << m_arguments.join(" ");
    pppd->start( m_arguments.join(" ") );
    
    if ( !pppd->waitForFinished( 90000 ) )
    {
        writeLog( "ConnectThread::run(): pppd->waitForFinished() returned false, killing the process..." );
        pppd->kill();

        disconnect( pppd, SIGNAL( lineAvailable( QByteArray ) ), this, SIGNAL( debugMsg( QByteArray ) ) );
        disconnect( pppd, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( pppdProcessError( QProcess::ProcessError ) ) );

        delete pppd;

        pppd = new MyProcess( 0 );
        pppd->setEnvironment( m_env );
        pppd->moveToThread( this );

        connect( pppd, SIGNAL( lineAvailable( QByteArray ) ), this, SIGNAL( debugMsg( QByteArray ) ) );
        connect( pppd, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( pppdProcessError( QProcess::ProcessError ) ) );
    }

    writeLog( "ConnectThread::run(): finished." );
}

void ConnectThread::prepare_nas_iface()
{
    writeLog( "ConnectThread::prepare_nas_iface(): called." );

    QNetworkInterface nas = QNetworkInterface::interfaceFromName( nas_iface );
    if ( nas.isValid() )
    {
        if ( nas.flags().testFlag( QNetworkInterface::IsUp ) )
        {
            writeLog( "ConnectThread::prepare_nas_iface(): interface " + nas_iface +" is up." );
        }
        else
        {
            writeLog( "ConnectThread::prepare_nas_iface(): interface " + nas_iface + " is down." );
            QProcess ifconfig;
            connect( &ifconfig, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( ifconfigProcessError( QProcess::ProcessError ) ) );

            ifconfig.start( "ifconfig", QStringList() << nas_iface << "up" );
            if ( !ifconfig.waitForFinished( 30000 ) )
            {
                ifconfig.terminate();
                writeLog( "ConnectThread::prepare_nas_iface(): ifconfig " + nas_iface + " up: process timeout." );
            }
            else if ( ifconfig.exitCode() == 0 )
            {
                writeLog( "ConnectThread::prepare_nas_iface(): ifconfig " + nas_iface + " up: success." );
                QTimer::singleShot( 1500, this, SLOT( quit() ) );
                exec();
            }
            else
            {
                writeLog( "ConnectThread::prepare_nas_iface(): ifconfig " + nas_iface + " up: failed!" );
            }
        }
    }

    writeLog( "ConnectThread::prepare_nas_iface(): finished." );
}

void ConnectThread::prepare_zx_iface()
{
    writeLog( "ConnectThread::prepare_zx_iface(): called." );

    if (( zx_iface = findInterface( "zx" ) ) == "" )
        zx_iface = findInterface( "dsl", "dsl0" );

    QNetworkInterface dsl = QNetworkInterface::interfaceFromName( zx_iface );

    if ( dsl.isValid() )
    {
        if ( dsl.flags().testFlag( QNetworkInterface::IsUp ) )
        {
            writeLog( "ConnectThread::prepare_zx_iface(): interface " + zx_iface + " is up." );
        }
        else
        {
            writeLog( "ConnectThread::prepare_zx_iface(): interface " + zx_iface + " is down." );
            QProcess ifconfig;
            connect( &ifconfig, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( ifconfigProcessError( QProcess::ProcessError ) ) );
            ifconfig.moveToThread( this );
            ifconfig.start( "ifconfig", QStringList() << zx_iface << "up" );
            ifconfig.waitForFinished( 30000 );

            if ( ifconfig.exitCode() == 0 )
                writeLog( "ConnectThread::prepare_zx_iface(): ifconfig " + zx_iface + " up: success." );
            else
                writeLog( "ConnectThread::prepare_zx_iface(): ifconfig " + zx_iface + " up: failed!" );
        }
    }

    writeLog( "ConnectThread::prepare_zx_iface(): finished." );
}

void ConnectThread::prepareBridging()
{
    writeLog( "ConnectThread::prepareBridging(): called." );

    QProcess proc;
    connect( &proc, SIGNAL( error( QProcess::ProcessError ) ), this, SLOT( brctlProcessError( QProcess::ProcessError ) ) );
    proc.moveToThread( this );
    proc.setEnvironment( m_env );
    proc.execute( "modprobe", QStringList() << "br2684" );

    QTimer::singleShot( 1000, this, SLOT( quit() ) );
    exec();

    const QString VpiVci = m_settings->value( "vpi", 0 ).toString() + "." + m_settings->value( "vci", 0 ).toString();
    writeLog( "ConnectThread::prepareBridging(): VPI.VCI: " + VpiVci );

    QString br2684ctl = UBUDSL_PREFIX + "/share/ubudsl/bin/br2684ctl";
    if ( !QFile::exists( br2684ctl ) )
        br2684ctl = "br2684ctl";

    proc.start( br2684ctl, QStringList() << "-b" << "-c" << "0" << "-a" << VpiVci );
    proc.waitForFinished( 5000 );
    qDebug() << proc.pid();

    if ( proc.exitCode() == 0 )
        writeLog( "ConnectThread::prepareBridging(): br2684ctl: success." );
    else
        writeLog( "ConnectThread::prepareBridging(): br2684ctl: failed!" );

    QTimer::singleShot( 2000, this, SLOT( quit() ) );
    exec();

    nas_iface = findInterface( "nas", "nas0" );

    //proc.start("ip", QStringList() << "addr" << "add" << "192.168.0.1/32" << "dev" << "nas0" );
    proc.start( "ifconfig", QStringList() << nas_iface << "192.168.2.1" <<  "netmask" << "255.255.255.0"  );
    proc.waitForFinished( 5000 );

    if ( proc.exitCode() == 0 )
        writeLog( "ConnectThread::prepareBridging(): ifconfig: success." );
    else
        writeLog( "ConnectThread::prepareBridging(): ifconfig: failed!" );

    QTimer::singleShot( 2000, this, SLOT( quit() ) );
    exec();

    writeLog( "ConnectThread::prepareBridging(): finished." );
}

bool ConnectThread::waitForSynchronisation()
{
    static int counter = 0;

    if ( getModemState() == Synchronised )
    {
        writeLog( "ConnectThread::waitForSynchronisation(): modem synchronised." );

        counter = 0;
        return true;
    }

    if ( counter > 6 )
    {
        writeLog( "ConnectThread::waitForSynchronisation(): counter > 6, resetting and returning." );

        counter = 0;
        m_returnState = ReturnState::SynchronisationFailed;

        return false;
    }

    if ( counter == 0 )
    {
        writeLog( "ConnectThread::waitForSynchronisation(): counter == 0, emitting the synchronising signal." );

        emit synchronising();
    }

    QTimer::singleShot( 5000, this, SLOT( quit() ) );
    exec();

    counter++;

    return waitForSynchronisation();
}

ReturnState::ReturnState ConnectThread::getReturnState()
{
    return m_returnState;
}

void ConnectThread::setModemState( int state )
{
    m_modemState = state;
}

void ConnectThread::processError( const QString process, QProcess::ProcessError &error )
{
    QString msg = process + " error: ";

    switch ( error )
    {
        case QProcess::FailedToStart:
            msg += "failed to start...";
            break;
        case QProcess::Crashed:
            msg += "crashed...";
            break;
        case QProcess::Timedout:
            msg += "failed to start.";
            break;
        case QProcess::WriteError:
            msg += "write error...";
            break;
        case QProcess::ReadError:
            msg += "read error...";
            break;
        case QProcess::UnknownError:
            msg += "unknown error...";
            break;
    }

    writeLog( "ConnectThread::processError(): " + msg );
}

